/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

import java.util.Iterator;
import java.util.NoSuchElementException;

import datatypes.GroupIterator;

/**
 * An implementation of the GroupIterator interface used to iterate over
 * newsgroup names.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.2, 04/01/03
 */
class BasicGroupIterator implements GroupIterator {
    private Iterator iterator;

    /**
     * Creates a new SimpleGroupIterator object.
     *
     * @param iterator The iterator containing all the group names.
     */
    protected BasicGroupIterator(Iterator iterator) {
        this.iterator = iterator;
    }

    /**
     * Queries if there are more groups in this iterator.
     *
     * @return true if more groups exist; false otherwise.
     */
    public boolean hasMoreGroups() {
        return this.iterator.hasNext();
    }

    /**
     * Gets the next group name in this iterator.
     *
     * @return The next group name in this iterator.
     *
     * @throws NoSuchElementException if there are no more elements in this
     *         iteration.
     */
    public String nextGroup() throws NoSuchElementException {
        return (String)this.iterator.next();
    }
}
