/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

import java.util.Iterator;
import java.util.NoSuchElementException;

import datatypes.MessageIDIterator;

/**
 * An implementation of the MessageIDIterator interface for iterating over a
 * set of Message IDs.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.1, 03/28/03
 */
class BasicMessageIDIterator implements MessageIDIterator {
    private Iterator iterator;

    /**
     * Creates a new SimpleMessageIDIterator object.
     *
     * @param iterator An Iterator containing all the message IDs in the
     *        map.
     */
    protected BasicMessageIDIterator(Iterator iterator) {
        this.iterator = iterator;
    }

    /**
     * Queries if this iterator has more message IDs.
     *
     * @return true if more message IDs exist; false otherwise.
     */
    public boolean hasMoreMessageIDs() {
        return this.iterator.hasNext();
    }

    /**
     * Gets the next message ID in this iterator.
     *
     * @return The next message ID.
     *
     * @throws NoSuchElementException if no more elements are in this
     *         iterator.
     */
    public String nextMessageID() throws NoSuchElementException {
        return (String)this.iterator.next();
    }
}
