/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datastructures;

import datatypes.UserSettings;

/**
 * An implementation of the UserSettings interface.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.3, 04/11/03
 */
public class BasicUserSettings implements UserSettings {
    private String hostname = DEFAULT_HOSTNAME;
    private int port = DEFAULT_PORT;
    private String userName = DEFAULT_NAME;
    private String userEmail = DEFAULT_EMAIL;
    private String userSignature = "";
    private static final BasicUserSettings INSTANCE = new BasicUserSettings();

    /**
     * The default newserver hostname.
     */
    public static final String DEFAULT_HOSTNAME = "127.0.0.1";

    /**
     * The default newserver port.
     */
    public static final int DEFAULT_PORT = 1515;

    /**
     * The default user's name.
     */
    public static final String DEFAULT_NAME = "John Doe";

    /**
     * The default user's email address.
     */
    public static final String DEFAULT_EMAIL = "john.doe@xsoap.com";

    /**
     * Creates a new BasicUserSettings object.
     */
    private BasicUserSettings() {
        // we are a singleton, disallow foreign creation...
    }

    /**
     * Gets an instance of UserSettings.
     *
     * @return a reference to a UserSettings object.
     */
    public static UserSettings getInstance() {
        return (UserSettings)INSTANCE;
    }

    /**
     * Gets the news server's hostname.
     *
     * @return The news server's hostname.
     */
    public String getHostname() {
        return this.hostname;
    }

    /**
     * Sets the news server's hostname.
     *
     * @param hostname The new news server's hostname.
     */
    public void setHostname(String hostname) {
        this.hostname = hostname;
    }

    /**
     * Gets the port the news server is on.
     *
     * @return The new server's port.
     */
    public int getPort() {
        return this.port;
    }

    /**
     * Sets the port the news server is on.
     *
     * @param port The new news server's port.
     */
    public void setPort(int port) {
        this.port = port;
    }

    /**
     * Gets the user's name.
     *
     * @return The user's name.
     */
    public String getUserName() {
        return this.userName;
    }

    /**
     * Sets the user's name.
     *
     * @param userName The new user's name.
     */
    public void setUserName(String userName) {
        this.userName = userName;
    }

    /**
     * Gets the user's email address.
     *
     * @return The user's email address.
     */
    public String getUserEmail() {
        return this.userEmail;
    }

    /**
     * Sets the user's email address.
     *
     * @param userEmail The new user's email address.
     */
    public void setUserEmail(String userEmail) {
        this.userEmail = userEmail;
    }

    /**
     * Gets the user's signature.
     *
     * @return The user's signature.
     */
    public String getUserSignature() {
        return this.userSignature;
    }

    /**
     * Sets the user's signature.
     *
     * @param userSignature The new user's signature.
     */
    public void setUserSignature(String userSignature) {
        this.userSignature = userSignature;
    }
}
