/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datatypes;

import java.util.Date;

import org.gjt.xpp.XmlNode;

/**
 * An interface for a news item.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.2, 04/11/03
 */
public interface NewsItem {
    /**
     * Gets the newsgroup this news item is for.
     *
     * @return The newsgroup.
     */
    public String getGroup();

    /**
     * Gets the message ID of this news item.
     *
     * @return The message ID.
     */
    public String getMessageID();

    /**
     * Gets the poster of this message.
     *
     * @return The poster of this message.
     */
    public String getFrom();

    /**
     * Gets the header of this message.
     *
     * @return The message header.
     */
    public String getHeader();

    /**
     * Gets the body of this message.
     *
     * @return The message body.
     */
    public String getBody();

    /**
     * Gets the date this message was posted.
     *
     * @return The message's post date.
     */
    public Date getDate();

    /**
     * Sets the date of this message.
     *
     * @param date The new date of this message.
     */
    public void setDate(Date date);

    /**
     * Creates an XmlNode representation of this object.
     *
     * @return The XmlNode representation.
     */
    public XmlNode toXmlNode();
}
