/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package datatypes;

/**
 * An interface for managine user settings.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.1, 04/09/03
 */
public interface UserSettings {
    /**
     * Gets the news server's hostname.
     *
     * @return The news server's hostname.
     */
    public String getHostname();

    /**
     * Sets the news server's hostname.
     *
     * @param hostname The new news server's hostname.
     */
    public void setHostname(String hostname);

    /**
     * Gets the port the news server is on.
     *
     * @return The new server's port.
     */
    public int getPort();

    /**
     * Sets the port the news server is on.
     *
     * @param port The new news server's port.
     */
    public void setPort(int port);

    /**
     * Gets the user's name.
     *
     * @return The user's name.
     */
    public String getUserName();

    /**
     * Sets the user's name.
     *
     * @param userName The new user's name.
     */
    public void setUserName(String userName);

    /**
     * Gets the user's email address.
     *
     * @return The user's email address.
     */
    public String getUserEmail();

    /**
     * Sets the user's email address.
     *
     * @param userEmail The new user's email address.
     */
    public void setUserEmail(String userEmail);

    /**
     * Gets the user's signature.
     *
     * @return The user's signature.
     */
    public String getUserSignature();

    /**
     * Sets the user's signature.
     *
     * @param userSignature The new user's signature.
     */
    public void setUserSignature(String userSignature);
}
