/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package protocol;

/**
 * A class for building Nntp servers.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.1, 04/11/03
 */
public abstract class AbstractNntp implements Nntp {
    /**
     * Gets an instance of a root server.
     *
     * @return The Nntp service interface.
     *
     * @throws ServerBindException if the server wasn't able to be created.
     */
    public static final Nntp getInstance() throws ServerBindException {
        // create the server
        BasicNntp server = new BasicNntp();

        // try to initalize the server
        server.initServer();

        // return the server instance
        return server;
    }

    /**
     * Gets an instance of a child server.
     *
     * @param parentServer The parent of this child server.
     *
     * @return The Nntp service interface.
     *
     * @throws ServerBindException if the server wasn't able to be created.
     */
    public static final Nntp getInstance(String parentServer)
                                  throws ServerBindException {
        // create the server
        ChildNntp server = new ChildNntp(parentServer);

        // try to initialize the server
        server.initServer(parentServer);

        return server;
    }
}
