/*
 * Copyright (C) 2003 John David Ratliff
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;

import java.text.SimpleDateFormat;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.JApplet;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;

import javax.swing.border.BevelBorder;
import javax.swing.border.EtchedBorder;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import datastructures.BasicGroupMap;
import datastructures.BasicUserSettings;
import datastructures.XMLParseException;
import datastructures.XMLParser;

import datatypes.GroupIterator;
import datatypes.GroupMap;
import datatypes.MessageIDIterator;
import datatypes.NewsItem;
import datatypes.UserSettings;

import driver.Debug;

import org.gjt.xpp.XmlNode;

import protocol.Nntp;

import soaprmi.RemoteException;

import soaprmi.soaprpc.SoapServices;

/**
 * The client applet which contains the client interface. GUI elements were
 * adapted from code produced by NetBeans 3.4.1.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.6, 04/12/03
 */
public class ClientApplet extends JApplet {
    private JLabel messageID;
    private JLabel date;
    private JList newsgroupList;
    private JTextArea body;
    private JLabel group;
    private JList messageList;
    private JLabel header;
    private JLabel from;
    private JMenuItem newsPost;
    private JMenuItem newsRefresh;
    private Nntp server;
    private GroupMap groups;
    private long lastChecked;
    private final SimpleDateFormat dateFormatter = new SimpleDateFormat("EEE, dd.MMM.yyyy hh:mm:ss a");
    private UserSettings settings = BasicUserSettings.getInstance();
    private static Frame frame = new JFrame();
    private static final ClientApplet INSTANCE = new ClientApplet();

    /**
     * Creates a new ClientApplet object.
     */
    private ClientApplet() {
        // we are a singleton - disallow foreign creation
    }

    /**
     * Gets an instance of this applet with a generic frame for dialog
     * display.
     *
     * @return A JApplet reference to this applet.
     */
    public static final JApplet getInstance() {
        return (JApplet)INSTANCE;
    }

    /**
     * Gets an instance of this applet.
     *
     * @param frame The frame to act as the parent of dialogs.
     *
     * @return A JApplet reference to this applet.
     */
    public static final JApplet getInstance(Frame frame) {
        ClientApplet.frame = frame;
        return (JApplet)INSTANCE;
    }

    /**
     * Initializes the main menu bar.
     */
    private void initMenuBar() {
        JMenuBar mainMenuBar = new JMenuBar();
        JMenu fileMenu = new JMenu();
        JMenuItem fileExit = new JMenuItem();
        JMenu editMenu = new JMenu();
        JMenuItem editPreferences = new JMenuItem();
        JMenu newsMenu = new JMenu();
        JMenu helpMenu = new JMenu();
        JMenuItem helpAbout = new JMenuItem();

        this.newsPost = new JMenuItem();
        this.newsRefresh = new JMenuItem();

        fileMenu.setMnemonic('f');
        fileMenu.setText("File");
        fileExit.setMnemonic('x');
        fileExit.setText("Exit");
        fileExit.setToolTipText("Exits the program");
        fileMenu.add(fileExit);

        fileExit.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    int choice = JOptionPane.showConfirmDialog(frame,
                                                               "Really exit?",
                                                               "JOC - Exit Confirmation",
                                                               JOptionPane.YES_NO_OPTION,
                                                               JOptionPane.QUESTION_MESSAGE);

                    if (choice == JOptionPane.YES_OPTION) {
                        System.exit(0);
                    }
                }
            });

        mainMenuBar.add(fileMenu);

        editMenu.setMnemonic('e');
        editMenu.setText("Edit");
        editPreferences.setMnemonic('p');
        editPreferences.setText("Preferences");
        editPreferences.setToolTipText("Sets the program preferences.");
        editMenu.add(editPreferences);

        editPreferences.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    String oldServer = settings.getHostname();
                    int oldPort = settings.getPort();

                    PreferencesDialog dialog = new PreferencesDialog(settings,
                                                                     frame,
                                                                     true);
                    int x = (frame.getWidth() / 2) -
                            (dialog.getWidth() / 2);
                    int y = (frame.getHeight() / 2) -
                            (dialog.getHeight() / 2);
                    dialog.setLocation(x, y);
                    dialog.show();

                    if (dialog.hasUpdated()) {
                        settings = dialog.getSettings();

                        // try to change the server
                        if ((!oldServer.equals(settings.getHostname()) ||
                            (oldPort != settings.getPort())) ||
                            (groups == null)) {
                            if (!changeServer()) {
                                settings.setHostname(oldServer);
                                settings.setPort(oldPort);
                                JOptionPane.showMessageDialog(frame,
                                                              "Unable to connect to server at " +
                                                              getHostname(),
                                                              "Server connection error.",
                                                              JOptionPane.ERROR_MESSAGE);
                            }
                        }
                    }
                }
            });

        mainMenuBar.add(editMenu);

        newsMenu.setMnemonic('n');
        newsMenu.setText("News");
        newsPost.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,
                                                       InputEvent.CTRL_MASK));
        newsPost.setMnemonic('p');
        newsPost.setText("Post");
        newsPost.setToolTipText("Post a message to the server.");
        newsPost.setEnabled(false);
        newsMenu.add(newsPost);

        newsPost.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    ComposeDialog message = new ComposeDialog(server,
                                                              settings,
                                                              (String)newsgroupList.getSelectedValue(),
                                                              frame, false);
                    message.show();
                }
            });

        newsRefresh.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U,
                                                          InputEvent.CTRL_MASK));
        newsRefresh.setMnemonic('u');
        newsRefresh.setText("Update News");
        newsRefresh.setToolTipText("Gets all new messages on the server.");
        newsRefresh.setEnabled(false);
        newsMenu.add(newsRefresh);

        newsRefresh.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    // retrieve new messages from server
                    getNewServerMessages();
                }
            });

        mainMenuBar.add(newsMenu);

        helpMenu.setMnemonic('h');
        helpMenu.setText("Help");
        helpAbout.setMnemonic('a');
        helpAbout.setText("About");
        helpAbout.setToolTipText("Displays the program version and author information.");
        helpMenu.add(helpAbout);

        helpAbout.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    JOptionPane.showMessageDialog(frame,
                                                  "JOC - The Java Open Client version 1.0.0\n" +
                                                  "Copyright (C) 2003 John David Ratliff, Yohei Kaneko",
                                                  "About JOC...",
                                                  JOptionPane.INFORMATION_MESSAGE);
                }
            });

        mainMenuBar.add(helpMenu);

        this.setJMenuBar(mainMenuBar);
    }

    /**
     * Initializes the swing components.
     */
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        // create the swing components
        this.newsgroupList = new JList();
        this.group = new JLabel();
        this.from = new JLabel();
        this.date = new JLabel();
        this.header = new JLabel();
        this.body = new JTextArea();
        this.messageID = new JLabel();
        this.messageList = new JList();

        JPanel messagePanel = new JPanel();
        JPanel displayPanel = new JPanel();
        JLabel groupLabel = new JLabel();
        JLabel fromLabel = new JLabel();
        JLabel dateLabel = new JLabel();
        JLabel headerLabel = new JLabel();
        JLabel messageIDLabel = new JLabel();
        JScrollPane messageScroller = new JScrollPane();
        JScrollPane bodyScroller = new JScrollPane();

        this.getContentPane().setLayout(new GridBagLayout());

        this.setName("JOC Applet");
        this.newsgroupList.setBorder(new EtchedBorder(EtchedBorder.RAISED,
                                                      Color.white,
                                                      Color.black));
        this.newsgroupList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        this.newsgroupList.setPreferredSize(new Dimension(100, 100));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(0, 0, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.weightx = 0.3;
        gridBagConstraints.weighty = 1.0;
        this.getContentPane().add(this.newsgroupList, gridBagConstraints);

        this.newsgroupList.addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent event) {
                    // retrieve new messages and build the list
                    getNewServerMessages();
                    updateMessageList();
                }
            });

        messagePanel.setLayout(new BorderLayout());

        displayPanel.setLayout(new GridBagLayout());

        groupLabel.setText("Group: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.insets = new Insets(5, 0, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        displayPanel.add(groupLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.insets = new Insets(5, 0, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        displayPanel.add(this.group, gridBagConstraints);

        fromLabel.setText("From: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        displayPanel.add(fromLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        displayPanel.add(this.from, gridBagConstraints);

        dateLabel.setText("Date: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        displayPanel.add(dateLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        displayPanel.add(this.date, gridBagConstraints);

        headerLabel.setText("Subject: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        displayPanel.add(headerLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        displayPanel.add(this.header, gridBagConstraints);

        bodyScroller.setPreferredSize(new Dimension(200, 200));
        this.body.setLineWrap(true);
        this.body.setEditable(false);
        this.body.setBorder(new BevelBorder(BevelBorder.RAISED,
                                            Color.darkGray, Color.lightGray,
                                            Color.blue, Color.black));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 0, 0, 0);
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        bodyScroller.setViewportView(this.body);
        displayPanel.add(bodyScroller, gridBagConstraints);

        messageIDLabel.setText("Identifier: ");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        displayPanel.add(messageIDLabel, gridBagConstraints);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        displayPanel.add(this.messageID, gridBagConstraints);

        messagePanel.add(displayPanel, BorderLayout.CENTER);

        messageScroller.setPreferredSize(new Dimension(200, 200));
        this.messageList.setBorder(new EtchedBorder(Color.white, Color.black));
        this.messageList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        messageScroller.setViewportView(this.messageList);

        this.messageList.addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent event) {
                    String selector = (String)messageList.getSelectedValue();

                    if (selector != null) {
                        int start = selector.lastIndexOf('<') + 1;
                        int end = selector.length() - 1;

                        viewMessage(selector.substring(start, end));
                    }
                }
            });

        messagePanel.add(messageScroller, BorderLayout.NORTH);

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.anchor = GridBagConstraints.EAST;
        gridBagConstraints.weightx = 0.7;
        gridBagConstraints.weighty = 1.0;
        this.getContentPane().add(messagePanel, gridBagConstraints);

        // build the menubar
        this.initMenuBar();
    }

    /**
     * Gets the new messages from the server and caches them locally.
     */
    private void getNewServerMessages() {
        try {
            // record the time just prior to asking for new news.
            long beforeTime = System.currentTimeMillis();

            // get the new news and parse it into an iterator
            XmlNode xmlNewNews = server.newNews(lastChecked);
            MessageIDIterator iterator = XMLParser.parseNewNews(xmlNewNews);

            // add all the new messages
            while (iterator.hasMoreMessageIDs()) {
                // get the next message and parse it into a NewsItem
                String messageID = iterator.nextMessageID();

                // confirm we didn't add this message already...
                if (!groups.containsMessage(messageID)) {
                    XmlNode xmlNewsItem = server.getMessage(messageID);
                    NewsItem message = XMLParser.parseNewsItem(xmlNewsItem);

                    // add the message to the local repository
                    groups.postMessage(message);

                    if (Debug.DEBUG_MODE) {
                        System.out.println("ClientApplet: adding message " +
                                           messageID);
                        System.out.println("ClientApplet: at time index " +
                                           new Date());
                    }
                }
            }

            // update the time last checked
            lastChecked = beforeTime;

            // update the current message list
            updateMessageList();
        } catch (XMLParseException e) {
            // this should really never happen
            e.printStackTrace();
        } catch (RemoteException e) {
            // this should only happen if a network layer
            // error occurs. i.e., rarely.
            e.printStackTrace();
        }
    }

    /**
     * Updates the message in the current message list.
     */
    private void updateMessageList() {
        String group = (String)this.newsgroupList.getSelectedValue();

        if (group != null) {
            // assuming they have selected a group...
            // get all messages in a group
            MessageIDIterator iterator = groups.getMessages(group);

            // create a temp array
            List list = new ArrayList();

            while (iterator.hasMoreMessageIDs()) {
                // get the next message
                String messageID = iterator.nextMessageID();
                NewsItem message = groups.getMessage(messageID);

                // create the message selector
                String selector = message.getHeader() + " <" + messageID +
                                  ">";
                list.add(selector);
            }

            // set the message list
            this.messageList.setListData(list.toArray());
        }
    }

    /**
     * Views a selected message.
     *
     * @param messageID The message to view.
     */
    private void viewMessage(String messageID) {
        if (Debug.DEBUG_MODE) {
            System.out.println("ClientApplet: displaying message " +
                               messageID);
        }

        NewsItem message = this.groups.getMessage(messageID);

        this.group.setText(message.getGroup());
        this.from.setText(message.getFrom());
        this.date.setText(this.dateFormatter.format(message.getDate()));
        this.messageID.setText(messageID);
        this.header.setText(message.getHeader());
        this.body.setText(message.getBody());
        this.body.setCaretPosition(0);
    }

    /**
     * Gets a fully qualified server name from the current settings.
     *
     * @return The server hostname.
     */
    private String getHostname() {
        return "http://" + this.settings.getHostname() + ":" +
               this.settings.getPort();
    }

    /**
     * Queries if a server is running by ping-ing it.
     *
     * @param server The server to ping.
     *
     * @return true if the server is running; false otherwise.
     */
    private boolean isRunning(String server) {
        try {
            // get the parent service interface
            SoapServices soap = SoapServices.getDefault();
            Nntp nntp = (Nntp)soap.createStartpoint(server,
                                                    new Class[] {Nntp.class});

            // ping to verify the 'alive-ness'
            nntp.ping();
        } catch (RemoteException e) {
            return false;
        }

        return true;
    }

    /**
     * Changes news servers.
     *
     * @return true if the change was successful; false otherwise.
     */
    private boolean changeServer() {
        String server = this.getHostname();

        if (this.isRunning(server)) {
            // throw away any old group information...
            this.groups = new BasicGroupMap();

            try {
                // get the parent service interface
                SoapServices soap = SoapServices.getDefault();
                this.server = (Nntp)soap.createStartpoint(server,
                                                          new Class[] {
                                                              Nntp.class
                                                          });

                // build the group list from the parent
                XmlNode xmlGroupList = this.server.list();

                GroupIterator iterator = XMLParser.parseGroupList(xmlGroupList);
                List list = new ArrayList();

                while (iterator.hasMoreGroups()) {
                    String group = iterator.nextGroup();
                    this.groups.addGroup(group);
                    list.add(group);
                }

                // set the groups list
                this.newsgroupList.setListData(list.toArray());

                // reset the last checked time
                this.lastChecked = 0;

                // enable posting and refresh
                this.newsPost.setEnabled(true);
                this.newsRefresh.setEnabled(true);

                if (Debug.DEBUG_MODE) {
                    System.out.println("ClientApplet: Connected to Nntp server at " +
                                       server);
                }
            } catch (RemoteException e) {
                // can't connect to a server? this will be rare...
                // we JUST pinged it.
                return false;
            } catch (XMLParseException e) {
                // this shouldn't happen either.
                return false;
            }

            return true;
        }

        return false;
    }

    /**
     * Initializes the applet.
     */
    public void init() {
        // not much to do here outside the component initialization
        this.initComponents();
    }

    /**
     * Starts the applet.
     */
    public void start() {
        if (Debug.DEBUG_MODE) {
            System.out.println("ClientApplet: starting the client...");
        }
    }
}
