/*
 * Copyright (C) 2003 John David Ratliff, Yohei Kaneko
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.util.regex.Pattern;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.WindowConstants;

import javax.swing.border.EtchedBorder;

import datatypes.UserSettings;

import driver.TCPConstants;

/**
 * A dialog for setting the application preferences. GUI elements were
 * adapted from code produced by NetBeans 3.4.1.
 *
 * @author John David Ratliff
 * @author Yohei Kaneko
 * @version 1.4, 04/11/03
 */
public class PreferencesDialog extends JDialog {
    private JTextField serverHostname;
    private JTextField serverPort;
    private JTextArea signature;
    private JTextField fullName;
    private JTextField email;
    private UserSettings settings;
    private boolean updated = false;

    /**
     * Creates a new PreferencesDialog object.
     *
     * @param settings The user setting's we are modifying.
     * @param parent The parent frame.
     * @param modal If this dialog should be modal.
     */
    protected PreferencesDialog(UserSettings settings, Frame parent,
                                boolean modal) {
        super(parent, modal);
        this.settings = settings;
        initComponents();
    }

    /**
     * Initializes the swing components.
     */
    private void initComponents() {
        GridBagConstraints gridBagConstraints;

        // create the Swing components
        this.serverHostname = new JTextField();
        this.serverPort = new JTextField();
        this.fullName = new JTextField();
        this.email = new JTextField();
        this.signature = new JTextArea();

        JLabel serverPreferencesLabel = new JLabel();
        JLabel serverHostnameLabel = new JLabel();
        JLabel serverPortLabel = new JLabel();
        JSeparator separator = new JSeparator();
        JLabel userPreferencesLabel = new JLabel();
        JLabel fullNameLabel = new JLabel();
        JLabel emailLabel = new JLabel();
        JLabel signatureLabel = new JLabel();
        JSeparator separator2 = new JSeparator();
        JPanel buttonPanel = new JPanel();
        JButton okButton = new JButton();
        JButton cancelButton = new JButton();
        JScrollPane signatureScroller = new JScrollPane();

        this.getContentPane().setLayout(new GridBagLayout());

        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.setTitle("JOC News Preferences");
        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent evt) {
                    closeDialog();
                }
            });

        serverPreferencesLabel.setFont(new Font("Dialog", 1, 14));
        serverPreferencesLabel.setForeground(new Color(0, 51, 153));
        serverPreferencesLabel.setText("Server Preferences");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.insets = new Insets(5, 5, 5, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        getContentPane().add(serverPreferencesLabel, gridBagConstraints);

        serverHostnameLabel.setText("Server Hostname: ");
        serverHostnameLabel.setToolTipText("The hostname of the news service");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new Insets(0, 15, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        getContentPane().add(serverHostnameLabel, gridBagConstraints);

        this.serverHostname.setText(this.settings.getHostname());
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1.0;
        this.getContentPane().add(this.serverHostname, gridBagConstraints);

        serverPortLabel.setText("Server Port: ");
        serverPortLabel.setToolTipText("The port the server is running on");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new Insets(0, 15, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        getContentPane().add(serverPortLabel, gridBagConstraints);

        this.serverPort.setText("" + this.settings.getPort());
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        getContentPane().add(this.serverPort, gridBagConstraints);

        separator.setForeground(new Color(51, 51, 51));
        separator.setPreferredSize(new Dimension(0, 1));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(5, 10, 10, 10);
        gridBagConstraints.weightx = 0.8;
        getContentPane().add(separator, gridBagConstraints);

        userPreferencesLabel.setFont(new Font("Dialog", 1, 14));
        userPreferencesLabel.setForeground(new Color(0, 51, 153));
        userPreferencesLabel.setText("User Preferences");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new Insets(0, 5, 5, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        getContentPane().add(userPreferencesLabel, gridBagConstraints);

        fullNameLabel.setText("Full Name: ");
        fullNameLabel.setToolTipText("Your full name, used to post messages quicker.");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 5;
        gridBagConstraints.insets = new Insets(0, 15, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        getContentPane().add(fullNameLabel, gridBagConstraints);

        this.fullName.setText(this.settings.getUserName());
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        getContentPane().add(this.fullName, gridBagConstraints);

        emailLabel.setText("Email Address: ");
        emailLabel.setToolTipText("Your email address, used to post messages quicker.");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 6;
        gridBagConstraints.insets = new Insets(0, 15, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        getContentPane().add(emailLabel, gridBagConstraints);

        this.email.setText(this.settings.getUserEmail());
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        getContentPane().add(this.email, gridBagConstraints);

        signatureLabel.setText("Signature: ");
        signatureLabel.setToolTipText("A signature which can be appended to all messages.");
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 7;
        gridBagConstraints.insets = new Insets(0, 15, 0, 0);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        getContentPane().add(signatureLabel, gridBagConstraints);

        signatureScroller.setPreferredSize(new Dimension(200, 200));
        this.signature.setLineWrap(true);
        this.signature.setText(this.settings.getUserSignature());
        signature.setBorder(new EtchedBorder(Color.white, Color.black));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 7;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        signatureScroller.setViewportView(this.signature);
        getContentPane().add(signatureScroller, gridBagConstraints);

        separator2.setForeground(new Color(51, 51, 51));
        separator2.setPreferredSize(new Dimension(0, 1));
        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 8;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets = new Insets(5, 10, 10, 10);
        getContentPane().add(separator2, gridBagConstraints);

        okButton.setMnemonic('o');
        okButton.setText("OK");
        okButton.setToolTipText("Saves these preferences");
        buttonPanel.add(okButton);

        okButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    validateAndClose();
                }
            });

        cancelButton.setMnemonic('c');
        cancelButton.setText("CANCEL");
        cancelButton.setToolTipText("Cancels the property settings");
        buttonPanel.add(cancelButton);

        cancelButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    closeDialog();
                }
            });

        gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridy = 9;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weighty = 0.5;
        getContentPane().add(buttonPanel, gridBagConstraints);

        pack();
    }

    /**
     * Validates the preferences fields and closes the dialog, if all fields
     * are valid.
     */
    private void validateAndClose() {
        String hostname = this.serverHostname.getText();
        String port = this.serverPort.getText();
        int serverPort = 0;
        String name = this.fullName.getText();
        String email = this.email.getText();
        String signature = this.signature.getText();
        boolean valid = true;
        Frame frame = (Frame)this.getParent();

        if (!Pattern.matches("[a-zA-Z0-9]+((\\.[a-zA-Z0-9]+)*\\.[a-zA-Z0-9]+)?",
                             hostname)) {
            // invalid hostname
            valid = false;

            JOptionPane.showMessageDialog(frame,
                                          "Server hostname must be of the form abc.xyz.foo, e.g. termite.cs",
                                          "Invalid Server Hostname",
                                          JOptionPane.ERROR_MESSAGE);
        } else if (!Pattern.matches(".+@[a-zA-Z0-9]+((\\.[a-zA-Z0-9]+)*\\.[a-zA-Z0-9]+)?",
                                    email)) {
            // invalid email
            valid = false;

            JOptionPane.showMessageDialog(frame,
                                          "Email address must be of the form abc@foo.com, e.g. jack@daniels.com",
                                          "Invalid Email Address",
                                          JOptionPane.ERROR_MESSAGE);
        } else {
            try {
                serverPort = Integer.parseInt(port);

                if ((serverPort < TCPConstants.MIN_PORT_NUMBER) ||
                    (serverPort > TCPConstants.MAX_PORT_NUMBER)) {
                    // invalid port
                    valid = false;
                }
            } catch (NumberFormatException e) {
                // invalid port
                valid = false;
            }

            if (!valid) {
                // invalid port
                JOptionPane.showMessageDialog(frame,
                                              "Port must be an integer in the range 1024-65535, e.g. 48526.",
                                              "Invalid Server Port",
                                              JOptionPane.ERROR_MESSAGE);
            }
        }

        if (valid) {
            this.settings.setHostname(hostname);
            this.settings.setPort(serverPort);
            this.settings.setUserName(name);
            this.settings.setUserEmail(email);
            this.settings.setUserSignature(signature);
            
            this.updated = true;

            // we can close now...
            closeDialog();
        }
    }

    /**
     * Gets the current settings as set by this dialog.
     *
     * @return The user settings.
     */
    public UserSettings getSettings() {
        return this.settings;
    }

    /**
     * Queries if this dialog updated the preferences.
     *
     * @return true if the preferences have changed; false otherwise.
     */
    public boolean hasUpdated() {
        return this.updated;
    }

    /**
     * Closes this dialog.
     */
    private void closeDialog() {
        setVisible(false);
        dispose();
    }
}
